#include <stdio.h>
#include <stdlib.h>

#define MODE_TEXT 0
#define MODE_BINARY 1

#define BUFSIZE 8192
char buf[BUFSIZE];

void fread_noerr(void *ptr, size_t size, size_t nmemb, FILE *stream){
	size_t rsize = fread(ptr, size, nmemb, stream);
	
	if(rsize < nmemb){
		fprintf(stderr, "file input error\n");
		exit(1);
	}
}

unsigned int fread_number(FILE *fp){
	unsigned int ret = 0;
	
	while(1){
		fread_noerr(buf, sizeof(char), 1, fp);
		if((buf[0] >= '0') && (buf[0] <= '9'))
			break;
	}
	
	do{
		ret = ret * 10 + (unsigned int)(buf[0] - '0');
		fread_noerr(buf, sizeof(char), 1, fp);
	}while((buf[0] >= '0') && (buf[0] <= '9'));
	
	return ret;
}

int main(int argc, char *argv[]){
	FILE *fp1, *fp2;
	int mode1, mode2;
	unsigned int i;
	unsigned int width1, height1, width2, height2;
	unsigned int rgbmax1, rgbmax2;
	
	if(argc < 3){
		fprintf(stderr, "usage: ppmdiff [ppmfile1] [ppmfile2]\n");
		return 1;
	}
	
	if(!(fp1 = fopen(argv[1], "rb"))){
		fprintf(stderr, "file %s not found\n", argv[1]);
		return 1;
	}
	
	if(!(fp2 = fopen(argv[2], "rb"))){
		fprintf(stderr, "file %s not found\n", argv[2]);
		fclose(fp1);
		return 1;
	}
	
	fread_noerr(buf, sizeof(char), 3, fp1);
	if((buf[0] == 'P') && (buf[1] == '3'))
		mode1 = MODE_TEXT;
	else if((buf[0] == 'P') && (buf[1] == '6'))
		mode1 = MODE_BINARY;
	
	fread_noerr(buf, sizeof(char), 3, fp2);
	if((buf[0] == 'P') && (buf[1] == '3'))
		mode2 = MODE_TEXT;
	else if((buf[0] == 'P') && (buf[1] == '6'))
		mode2 = MODE_BINARY;
	
	width1 = fread_number(fp1);
	height1 = fread_number(fp1);
	width2 = fread_number(fp2);
	height2 = fread_number(fp2);
	
	if((width1 != width2) || (height1 != height2)){
		printf("differs in picture size\n");
		fclose(fp2);
		fclose(fp1);
		return 0;
	}
	
	rgbmax1 = fread_number(fp1);
	rgbmax2 = fread_number(fp2);
	
	if(rgbmax1 != rgbmax2){
		printf("differs in rgbmax\n");
		fclose(fp2);
		fclose(fp1);
		return 0;
	}
	
	for(i = 0; i < width1 * height1; i++){
		unsigned int r1, g1, b1;
		unsigned int r2, g2, b2;
		
		if(mode1 == MODE_TEXT){
			r1 = fread_number(fp1);
			g1 = fread_number(fp1);
			b1 = fread_number(fp1);
		}
		else{
			fread_noerr(buf, sizeof(char), 1, fp1);
			r1 = (unsigned int)((unsigned char)(buf[0]));
			fread_noerr(buf, sizeof(char), 1, fp1);
			g1 = (unsigned int)((unsigned char)(buf[0]));
			fread_noerr(buf, sizeof(char), 1, fp1);
			b1 = (unsigned int)((unsigned char)(buf[0]));
		}
		
		if(mode2 == MODE_TEXT){
			r2 = fread_number(fp2);
			g2 = fread_number(fp2);
			b2 = fread_number(fp2);
		}
		else{
			fread_noerr(buf, sizeof(char), 1, fp2);
			r2 = (unsigned int)((unsigned char)(buf[0]));
			fread_noerr(buf, sizeof(char), 1, fp2);
			g2 = (unsigned int)((unsigned char)(buf[0]));
			fread_noerr(buf, sizeof(char), 1, fp2);
			b2 = (unsigned int)((unsigned char)(buf[0]));
		}
		
		if((r1 != r2) || (g1 != g2) || (b1 != b2)){
			unsigned int diff = (r1 - r2) * (r1 - r2) + (g1 - g2) * (g1 - g2) + (b1 - b2) * (b1 - b2);
			
			if(diff < 4)
				printf("  ");
			printf("diff %d in (%d, %d) : (%d, %d, %d) <--> (%d, %d, %d)\n", diff, i % width1, i / width1, r1, g1, b1, r2, g2, b2);
		}
	}
	
	fclose(fp2);
	fclose(fp1);
	
	return 0;
}
